@@ -3,6 +3,7 @@ package Rose::DB::Object::Metadata::Column;
 use strict;
 
 use Carp();
+use Scalar::Util();
 
 use Rose::DB::Object::Metadata::Util qw(:all);
 
@@ -14,7 +15,7 @@ use Rose::DB::Object::Util
     lazy_column_values_loaded_key);
 
 use Rose::DB::Object::Constants 
-  qw(STATE_IN_DB STATE_LOADING STATE_SAVING);
+  qw(STATE_IN_DB STATE_LOADING STATE_SAVING SAVING_FOR_LOAD);
 
 use Rose::Object::MakeMethods::Generic;
 use Rose::DB::Object::MakeMethods::Generic;
@@ -22,7 +23,7 @@ use Rose::DB::Object::MakeMethods::Generic;
 our $Triggers_Key      = 'triggers';
 our $Trigger_Index_Key = 'trigger_index';
 
-our $VERSION = '0.757';
+our $VERSION = '0.791';
 
 use overload
 (
@@ -32,10 +33,15 @@ use overload
 
 __PACKAGE__->add_default_auto_method_types('get_set');
 
-__PACKAGE__->add_common_method_maker_argument_names(qw(column default type hash_key));
+__PACKAGE__->add_common_method_maker_argument_names(qw(column default type db_type hash_key smart_modification undef_overrides_default));
 
 use Rose::Class::MakeMethods::Generic
 (
+  inheritable_scalar =>
+  [
+    'default_undef_overrides_default'
+  ],
+
   inheritable_hash =>
   [
     event_method_type  => { hash_key => 'event_method_types' },
@@ -46,31 +52,36 @@ use Rose::Class::MakeMethods::Generic
 
 __PACKAGE__->event_method_types
 (
-  inflate => [ qw(get_set get) ],
-  deflate => [ qw(get_set get) ],
-  on_load => [ qw(get_set set) ],
-  on_save => [ qw(get_set get) ],
-  on_set  => [ qw(get_set set) ],
-  on_get  => [ qw(get_set get) ],
+  inflate   => [ qw(get_set get) ],
+  deflate   => [ qw(get_set get) ],
+  on_load   => [ qw(get_set set) ],
+  on_save   => [ qw(get_set get) ],
+  on_set    => [ qw(get_set set) ],
+  on_get    => [ qw(get_set get) ],
+  lazy_load => [ qw(get_set get) ],
 );
 
 Rose::Object::MakeMethods::Generic->make_methods
 (
   { preserve_existing => 1 },
-  scalar => 
-  [
-    'alias',
-    'ordinal_position',
-    'parse_error',
-    __PACKAGE__->common_method_maker_argument_names,
-  ],
-
   boolean => 
   [
     'manager_uses_method',
     'is_primary_key_member',
     'not_null',
     'triggers_disabled',
+    'smart_modification',
+    'nonpersistent',
+  ],
+
+  scalar => 
+  [
+    'alias',
+    'error',
+    'ordinal_position',
+    'parse_error',
+    'remarks',
+    __PACKAGE__->common_method_maker_argument_names,
   ],
 );
 
@@ -98,6 +109,36 @@ __PACKAGE__->method_maker_info
   },
 );
 
+sub undef_overrides_default
+{
+  my($self) = shift;
+
+  if(@_)
+  {
+    return $self->{'undef_overrides_default'} = $_[0] ? 1 : 0;
+  }
+
+  return $self->{'undef_overrides_default'}
+    if(defined $self->{'undef_overrides_default'});
+
+  my $parent_default = $self->parent ? $self->parent->column_undef_overrides_default : undef;
+
+  return defined $parent_default ? $parent_default :  ref($self)->default_undef_overrides_default;
+}
+
+sub validate_specification
+{
+  my($self) = shift;
+
+  if($self->not_null && $self->{'undef_overrides_default'})
+  {
+    $self->error('True value for not_null attribute conflicts with true value for undef_overrides_default attribute.');
+    return 0;
+  }
+
+  return 1;
+}
+
 use constant ANY_DB => "\0ANY_DB\0";
 
 sub default_value_sequence_name
@@ -107,7 +148,18 @@ sub default_value_sequence_name
   my $db;
   $db = shift  if(UNIVERSAL::isa($_[0], 'Rose::DB'));
   my $parent = $self->parent;
-  my $db_id = $db ? $db->id : $parent ? $parent->init_db_id : ANY_DB;
+  my ($db_id, $error);
+
+  # Sometimes data source are not set up yet when this method
+  # is called.  Allow for failure, falling back to ANY_DB
+  TRY:
+  {
+    local $@;
+    eval { $db_id = $db ? $db->id : $parent ? $parent->init_db_id : ANY_DB };
+    $error = $@;
+  }
+
+  $db_id = ANY_DB if ($error);
 
   return $self->{'default_value_sequence_name'}{$db_id}  unless(@_);
 
@@ -202,7 +254,6 @@ sub made_method_type
     $self->{'accessor_method_name'} = $name;  
     $self->{'mutator_method_name'}  = $name;
     $self->{'rw_method_name'}       = $name;
-    $self->{'alias'} = $name;
   }  
   elsif($type eq 'get')
   {
@@ -327,7 +378,10 @@ sub init_with_dbi_column_info
   # We're doing this in Rose::DB::Object::Metadata::Auto now
   #$self->parent->db->refine_dbi_column_info($col_info);
 
-  $self->default($col_info->{'COLUMN_DEF'});
+  if(defined $col_info->{'COLUMN_DEF'})
+  {
+    $self->default($col_info->{'COLUMN_DEF'});
+  }
 
   if($col_info->{'NULLABLE'} == SQL_NO_NULLS)
   {
@@ -338,6 +392,17 @@ sub init_with_dbi_column_info
     $self->not_null(0);
   }
 
+  # DB-native type, if applicable
+  if(my $db_type = $col_info->{'RDBO_DB_TYPE'})
+  {
+    $self->db_type($db_type);
+  }
+
+  if($col_info->{'REMARKS'})
+  {
+    $self->remarks($col_info->{'REMARKS'});
+  }
+
   $self->ordinal_position($col_info->{'ORDINAL_POSITION'} || 0);
 
   $self->default_value_sequence_name($col_info->{'rdbo_default_value_sequence_name'});
@@ -356,7 +421,7 @@ sub perl_column_definition_attributes
     if($attr =~ /^(?: name(?:_sql)? | is_primary_key_member | 
                   primary_key_position | method_name | method_code |
                   made_method_types | ordinal_position | select_sql |
-                  (?:builtin_)?triggers | 
+                  undef_overrides_default | (?:builtin_)?triggers | 
                   (?:builtin_)?trigger_index )$/x)
     {
       next ATTR;
@@ -364,6 +429,7 @@ sub perl_column_definition_attributes
 
     my $val = $self->can($attr) ? $self->$attr() : next ATTR;
 
+    no warnings 'uninitialized';
     if(($attr eq 'check_in' || $attr eq 'values') &&
        ref $val && ref $val eq 'ARRAY')
     {
@@ -375,15 +441,27 @@ sub perl_column_definition_attributes
       else
       {
         $val = perl_arrayref(array => $val, inline => 1);
-        $attr = 'values';
+        $attr = 'check_in';
       }
     }
+    elsif($attr eq 'smart_modification' && 
+          (($self->smart_modification == ref($self)->new->smart_modification) ||
+           ($self->parent && $self->smart_modification == $self->parent->default_smart_modification)))
+    {
+      next ATTR;
+    }
+    elsif($attr eq 'undef_overrides_default' && 
+          (($self->undef_overrides_default == ref($self)->new->undef_overrides_default) ||
+           ($self->parent && $self->undef_overrides_default == $self->parent->column_undef_overrides_default)))
+    {
+      next ATTR;
+    }
     elsif(!defined $val || ref $val || ($attr eq 'not_null' && !$self->not_null))
     {
       next ATTR;
     }
 
-    if($attr eq 'alias' && $val eq $self->name)
+    if($attr eq 'alias' && (!defined $val || $val eq $self->name))
     {
       next ATTR;
     }
@@ -433,14 +511,14 @@ sub perl_column_definition_attributes
     {
       my $method = $self->$attr();
 
-      my $ok = 0;
+      my $skip = 1;
 
       foreach my $type ($self->auto_method_types)
       {
-        $ok = 1  if($method eq $self->build_method_name_for_type($type));
+        $skip = 0  if($method ne $self->build_method_name_for_type($type));
       }
 
-      next ATTR  if($ok);
+      next ATTR  if($skip);
     }
 
     push(@attrs, $attr);
@@ -516,7 +594,7 @@ sub lazy
     }
 
     $self->{'lazy'} = 1;
-    $self->add_builtin_trigger(event => 'on_get',
+    $self->add_builtin_trigger(event => 'lazy_load',
                                name  => 'load_on_demand',
                                code  => $self->load_on_demand_on_get_code);
 
@@ -590,12 +668,13 @@ sub load_on_demand_on_set_code
 
 our %Trigger_Events =
 (
-  inflate => 1,
-  deflate => 1,
-  on_load => 1,
-  on_save => 1,
-  on_set  => 1,
-  on_get  => 1,
+  inflate   => 1,
+  deflate   => 1,
+  on_load   => 1,
+  on_save   => 1,
+  on_set    => 1,
+  on_get    => 1,
+  lazy_load => 1,
 );
 
 sub trigger_events { keys %Trigger_Events }
@@ -914,6 +993,8 @@ sub apply_method_triggers
   unshift(@{$on_get_code ||= []}, @$builtins)
     if($builtins = $self->builtin_triggers('on_get'));  
 
+  my $lazy_load_code = $self->builtin_triggers('lazy_load');
+
   my $key             = $self->hash_key;
   my $formatted_key   = column_value_formatted_key($key);
   my $is_inflated_key = column_value_is_inflated_key($key);
@@ -924,7 +1005,8 @@ sub apply_method_triggers
   {
     if($inflate_code || $deflate_code || 
        $on_load_code || $on_save_code ||
-       $on_set_code  || $on_get_code)
+       $on_set_code  || $on_get_code  ||
+       $lazy_load_code)
     {
       my $method = sub
       {
@@ -938,9 +1020,18 @@ sub apply_method_triggers
           {
             $self->{$is_inflated_key} = 0  unless($uses_formatted_key);
 
-            &$method_code; # magic call using current @_
+            my @ret;
 
-            unless($self->{'triggers_disabled'})
+            if(wantarray)
+            {            
+              @ret = &$method_code; # magic call using current @_
+            }
+            else
+            {
+              $ret[0] = &$method_code; # magic call using current @_
+            }
+
+            unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
             {
               local $self->{'triggers_disabled'} = 1;
 
@@ -952,14 +1043,25 @@ sub apply_method_triggers
                 }
               }
             }
+
+            return wantarray ? @ret : $ret[0];
           }
           else
           {
             $self->{$is_inflated_key} = 0  unless($uses_formatted_key);
 
-            &$method_code; # magic call using current @_
+            my @ret;
 
-            unless($self->{'triggers_disabled'})
+            if(wantarray)
+            {            
+              @ret = &$method_code; # magic call using current @_
+            }
+            else
+            {
+              $ret[0] = &$method_code; # magic call using current @_
+            }
+
+            unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
             {
               local $self->{'triggers_disabled'} = 1;
 
@@ -971,6 +1073,8 @@ sub apply_method_triggers
                 }
               }
             }
+
+            return wantarray ? @ret : $ret[0];
           }
         }
         else # getting
@@ -979,7 +1083,7 @@ sub apply_method_triggers
           # duplicated to save one measly function call.  So sue me.
           if($self->{STATE_SAVING()})
           {
-            unless($self->{'triggers_disabled'})
+            unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
             {
               local $self->{'triggers_disabled'} = 1;
 
@@ -1022,7 +1126,7 @@ sub apply_method_triggers
                 }
               }
 
-              if($on_save_code)
+              if($on_save_code && !$self->{SAVING_FOR_LOAD()})
               {
                 foreach my $code (@$on_save_code)
                 {
@@ -1035,10 +1139,18 @@ sub apply_method_triggers
           }
           else
           {
-            unless($self->{'triggers_disabled'})
+            unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
             {
               local $self->{'triggers_disabled'} = 1;
 
+              if($lazy_load_code)
+              {
+                foreach my $code (@$lazy_load_code)
+                {
+                  $code->($self);
+                }
+              }
+
               if($inflate_code)
               {
                 my $value;
@@ -1054,7 +1166,7 @@ sub apply_method_triggers
                   $key_was_defined = 0;
                   # Invoke built-in default and inflation code
                   # (The call must not be in void context)
-                  $value = $method_code->($self, @_[1 .. $#$_]);
+                  $value = $method_code->($self, @_[1 .. $#_]);
                 }
 
                 unless($self->{$is_inflated_key} && $key_was_defined)
@@ -1117,7 +1229,7 @@ sub apply_method_triggers
   }
   elsif($type eq 'get')
   {
-    if($inflate_code || $deflate_code || $on_save_code || $on_get_code)
+    if($inflate_code || $deflate_code || $on_save_code || $on_get_code || $lazy_load_code)
     {
       my $method = sub
       {
@@ -1125,7 +1237,7 @@ sub apply_method_triggers
 
         if($self->{STATE_SAVING()})
         {
-          unless($self->{'triggers_disabled'})
+          unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
           {
             local $self->{'triggers_disabled'} = 1;
 
@@ -1168,7 +1280,7 @@ sub apply_method_triggers
               }
             }
 
-            if($on_save_code)
+            if($on_save_code && !$self->{SAVING_FOR_LOAD()})
             {
               foreach my $code (@$on_save_code)
               {
@@ -1181,10 +1293,18 @@ sub apply_method_triggers
         }
         else
         {
-          unless($self->{'triggers_disabled'})
+          unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
           {
             local $self->{'triggers_disabled'} = 1;
 
+            if($lazy_load_code)
+            {
+              foreach my $code (@$lazy_load_code)
+              {
+                $code->($self);
+              }
+            }
+
             if($inflate_code)
             {
               my $value;
@@ -1200,7 +1320,7 @@ sub apply_method_triggers
                 $key_was_defined = 0;
                 # Invoke built-in default and inflation code
                 # (The call must not be in void context)
-                $value = $method_code->($self, @_[1 .. $#$_]);
+                $value = $method_code->($self, @_[1 .. $#_]);
               }
 
               unless($self->{$is_inflated_key} && $key_was_defined)
@@ -1272,9 +1392,18 @@ sub apply_method_triggers
         {
           $self->{$is_inflated_key} = 0  unless($uses_formatted_key);
 
-          &$method_code; # magic call using current @_
+          my @ret;
 
-          unless($self->{'triggers_disabled'})
+          if(wantarray)
+          {            
+            @ret = &$method_code; # magic call using current @_
+          }
+          else
+          {
+            $ret[0] = &$method_code; # magic call using current @_
+          }
+
+          unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
           {
             local $self->{'triggers_disabled'} = 1;
 
@@ -1286,14 +1415,25 @@ sub apply_method_triggers
               }
             }
           }
+
+          return wantarray ? @ret : $ret[0];
         }
         else
         {
           $self->{$is_inflated_key} = 0  unless($uses_formatted_key);
 
-          &$method_code; # magic call using current @_
+          my @ret;
 
-          unless($self->{'triggers_disabled'})
+          if(wantarray)
+          {            
+            @ret = &$method_code; # magic call using current @_
+          }
+          else
+          {
+            $ret[0] = &$method_code; # magic call using current @_
+          }
+
+          unless($column->{'triggers_disabled'} || $self->{'triggers_disabled'})
           {
             local $self->{'triggers_disabled'} = 1;
 
@@ -1305,6 +1445,8 @@ sub apply_method_triggers
               }
             }
           }
+
+          return wantarray ? @ret : $ret[0];
         }
       };
 
@@ -1329,7 +1471,7 @@ sub method_code
 
   if(@_)
   {
-    return $self->{'method_code'}{$type} = shift;
+    Scalar::Util::weaken($self->{'method_code'}{$type} = shift);
   }
 
   return $self->{'method_code'}{$type};
@@ -1340,12 +1482,25 @@ sub make_methods
   my($self) = shift;
 
   $self->SUPER::make_methods(@_);
+
+  # Check if we can fold all method type name attributes
+  # into an alias attribute.  We can do this if the accessor,
+  # mutator, and rw method names are all the same and are not
+  # the same as the column name.
+  no warnings 'uninitialized';
+  if(($self->accessor_method_name eq $self->mutator_method_name &&
+      $self->mutator_method_name eq $self->rw_method_name &&
+      $self->rw_method_name ne $self->name))
+  {
+    $self->alias($self->rw_method_name);
+  }
+
   $self->apply_triggers;
 }
 
 sub dbi_requires_bind_param { 0 }
 
-# It's improtant to return undef or a hashref, not an empty list
+# It's important to return undef or a hashref, not an empty list
 sub dbi_bind_param_attrs { undef } 
 
 1;
@@ -1500,6 +1655,12 @@ For performance reasons, none of the column classes bundled with L<Rose::DB::Obj
 
 Get or set the default list of L<auto_method_types|/auto_method_types>.  TYPES should be a list of column method types.  Returns the list of default column method types (in list context) or a reference to an array of the default column method types (in scalar context).  The default list contains only the "get_set" column method type.
 
+=item B<default_undef_overrides_default [BOOL]>
+
+Get or set the default value of the L<undef_overrides_default|/undef_overrides_default> attribute.  The default value is undef.
+
+This default only applies when the column does not have a parent metadata object or if the metadata object's L<column_undef_overrides_default|Rose::DB::Object::Metadata/column_undef_overrides_default> method returns undef.
+
 =back
 
 =head1 CONSTRUCTOR
@@ -1732,10 +1893,15 @@ Returns the name of the method used to set the column value.  This is a convenie
 
 Get or set the name of the column, not including the table name, username, schema, or any other qualifier.
 
+=item B<nonpersistent [BOOL]>
+
+Get or set a boolean flag that indicates whether or not the column 
+is L<non-persistent|Rose::DB::Object::Metadata/nonpersistent_columns>.
+
 =item B<not_null [BOOL]>
 
 Get or set a boolean flag that indicates whether or not the column 
-value can can be null.
+value can be null.
 
 =item B<parse_value DB, VALUE>
 
@@ -1745,6 +1911,10 @@ Parse and return a convenient Perl representation of VALUE.  What form this valu
 
 Get or set the column's ordinal position in the primary key.  Returns undef if the column is not part of the primary key.  Position numbering starts from 1.
 
+=item B<remarks [TEXT]>
+
+Get or set a text description of the column.
+
 =item B<rw_method_name>
 
 Returns the name of the method used to get or set the column value.  This is a convenient shortcut for:
@@ -1796,6 +1966,54 @@ Returns true if L<triggers|/TRIGGERS> are disabled for this column, false otherw
 
 Returns the (possibly abstract) data type of the column.  The default implementation returns "scalar".
 
+=item B<undef_overrides_default [BOOL]>
+
+Get or set a boolean value that indicates whether or not setting the column to an undef value overrides the column's L<default|/default> value.
+
+The default value of this attribute is determined by the parent L<metadata|Rose::DB::Object::Metadata> object's L<column_undef_overrides_default|Rose::DB::Object::Metadata/column_undef_overrides_default> method, or the column's L<default_undef_overrides_default|/default_undef_overrides_default> class method id the metadata object's L<column_undef_overrides_default|Rose::DB::Object::Metadata/column_undef_overrides_default> method returns undef, or if the column has no parent metadata object.
+
+Example: consider a L<Rose::DB::Object>-derived C<Person> class with a C<name> column set up like this:
+
+    package Person;
+    ...
+       columns =>
+       [
+         name => { type => 'varchar', default => 'John Doe' },
+         ...
+       ],
+    ...
+
+The following behavior is the same regardless of the setting of the L<undef_overrides_default|/undef_overrides_default> attribute for the C<name> column:
+
+    $p = Person->new;
+    print $p->name; # John Doe
+
+    $p->name('Larry Wall');
+    print $p->name; # Larry Wall
+
+If L<undef_overrides_default|/undef_overrides_default> is B<false> for the C<name> column, then this is the behavior of explicitly setting the column to undef:
+
+    $p->name(undef);
+    print $p->name; # John Doe
+
+If L<undef_overrides_default|/undef_overrides_default> is B<true> for the C<name> column, then this is the behavior of explicitly setting the column to undef:
+
+    $p->name(undef);
+    print $p->name; # undef
+
+The L<undef_overrides_default|/undef_overrides_default> attribute can be set directly on the column:
+
+    name => { type => 'varchar', default => 'John Doe', 
+              undef_overrides_default => 1 },
+
+or it can be set class-wide using the L<meta|Rose::DB::Object/meta> object's L<column_undef_overrides_default|Rose::DB::Object::Metadata/column_undef_overrides_default> attribute:
+
+    Person->meta->column_undef_overrides_default(1);
+
+or it can be set for all classes that use a given L<Rose::DB::Object::Metadata>-derived class using the L<default_column_undef_overrides_default|Rose::DB::Object::Metadata/default_column_undef_overrides_default> class method:
+
+    My::DB::Object::Metadata->default_column_undef_overrides_default(1);
+
 =back
 
 =head1 PROTECTED API
@@ -1824,10 +2042,10 @@ Returns the method maker method type for the column method type TYPE.
 
 =head1 AUTHOR
 
-John C. Siracusa (siracusa@mindspring.com)
+John C. Siracusa (siracusa@gmail.com)
 
-=head1 COPYRIGHT
+=head1 LICENSE
 
-Copyright (c) 2006 by John C. Siracusa.  All rights reserved.  This program is
+Copyright (c) 2010 by John C. Siracusa.  All rights reserved.  This program is
 free software; you can redistribute it and/or modify it under the same terms
 as Perl itself.
